#include "driver.h"
#include "cpu/m6809/m6809.h"
#include "machine/6821pia.h"

#define MAX_VRAM 6144
#define MAX_HIRES_VRAM	57600

extern int dragon_cart_inserted;
extern UINT8 *dragon_tape;
extern int dragon_tapesize;
extern UINT8 *dragon_rom;
extern unsigned char *RAM;
UINT8 *dragon_ram;
int coco3_hires;
int coco3_gimevhreg[8];
int coco3_bordercolor;

static void d_pia1_pb_w(int offset, int data);
static void d_pia1_pa_w(int offset, int data);
static int d_pia1_cb1_r(int offset);
static int d_pia0_ca1_r(int offset);
static int d_pia0_pa_r(int offset);
static int d_pia1_pa_r(int offset);
static void d_pia0_pb_w(int offset, int data);
static void d_pia1_cb2_w(int offset, int data);
static void d_pia0_cb2_w(int offset, int data);
static void d_pia1_ca2_w(int offset, int data);
static void d_pia0_ca2_w(int offset, int data);
static void d_pia0_irq_b(int state);

static void coco_pia1_ca2_w(int offset, int data);

static struct pia6821_interface dragon_pia_0_intf =
{
	/*inputs : A/B,CA/B1,CA/B2 */ d_pia0_pa_r, 0, d_pia0_ca1_r, 0, 0, 0,
	/*outputs: A/B,CA/B2	   */ 0, d_pia0_pb_w, d_pia0_ca2_w, d_pia0_cb2_w,
	/*irqs	 : A/B			   */ 0, d_pia0_irq_b
};

static struct pia6821_interface dragon_pia_1_intf =
{
	/*inputs : A/B,CA/B1,CA/B2 */ d_pia1_pa_r, 0, 0, d_pia1_cb1_r, 0, 0,
	/*outputs: A/B,CA/B2	   */ d_pia1_pa_w, d_pia1_pb_w, d_pia1_ca2_w, d_pia1_cb2_w,
	/*irqs	 : A/B			   */ 0, 0
};

static struct pia6821_interface coco_pia_1_intf =
{
	/*inputs : A/B,CA/B1,CA/B2 */ d_pia1_pa_r, 0, 0, d_pia1_cb1_r, 0, 0,
	/*outputs: A/B,CA/B2	   */ d_pia1_pa_w, d_pia1_pb_w, coco_pia1_ca2_w, d_pia1_cb2_w,
	/*irqs	 : A/B			   */ 0, 0
};

static UINT8 pia1_pb, sound_mux, tape_motor;
static UINT8 joystick_axis, joystick;
static int display_offset, d_dac;
static UINT8 *dirtybuffer, *ptape;
static int sam_vdg_mode, pia_vdg_mode;
static struct GfxElement *dfont;
static struct GfxElement *coco3font;

/***************************************************************************
  Start the video hardware emulation.
***************************************************************************/

struct GfxElement *build_dragon_font(void)
{
	static unsigned char fontdata8x12[] =
	{
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x02, 0x1a, 0x2a, 0x2a, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x08, 0x14, 0x22, 0x22, 0x3e, 0x22, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3c, 0x12, 0x12, 0x1c, 0x12, 0x12, 0x3c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x20, 0x20, 0x20, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3c, 0x12, 0x12, 0x12, 0x12, 0x12, 0x3c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3e, 0x20, 0x20, 0x3c, 0x20, 0x20, 0x3e, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3e, 0x20, 0x20, 0x3c, 0x20, 0x20, 0x20, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1e, 0x20, 0x20, 0x26, 0x22, 0x22, 0x1e, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x3e, 0x22, 0x22, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x08, 0x08, 0x08, 0x08, 0x08, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x02, 0x02, 0x02, 0x02, 0x22, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x24, 0x28, 0x30, 0x28, 0x24, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x3e, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x36, 0x2a, 0x2a, 0x22, 0x22, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x32, 0x2a, 0x26, 0x22, 0x22, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3e, 0x22, 0x22, 0x22, 0x22, 0x22, 0x3e, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3c, 0x22, 0x22, 0x3c, 0x20, 0x20, 0x20, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x22, 0x22, 0x2a, 0x24, 0x1a, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3c, 0x22, 0x22, 0x3c, 0x28, 0x24, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x10, 0x08, 0x04, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3e, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x14, 0x14, 0x08, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x2a, 0x2a, 0x36, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x22, 0x14, 0x08, 0x14, 0x22, 0x22, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x22, 0x22, 0x14, 0x08, 0x08, 0x08, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3e, 0x02, 0x04, 0x08, 0x10, 0x20, 0x3e, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x38, 0x20, 0x20, 0x20, 0x20, 0x20, 0x38, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x20, 0x20, 0x10, 0x08, 0x04, 0x02, 0x02, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x0e, 0x02, 0x02, 0x02, 0x02, 0x02, 0x0e, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x08, 0x1c, 0x2a, 0x08, 0x08, 0x08, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x08, 0x10, 0x3e, 0x10, 0x08, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x00, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x14, 0x14, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x14, 0x14, 0x36, 0x00, 0x36, 0x14, 0x14, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x08, 0x1e, 0x20, 0x1c, 0x02, 0x3c, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x32, 0x32, 0x04, 0x08, 0x10, 0x26, 0x26, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x10, 0x28, 0x28, 0x10, 0x2a, 0x24, 0x1a, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x08, 0x10, 0x20, 0x20, 0x20, 0x10, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x08, 0x04, 0x02, 0x02, 0x02, 0x04, 0x08, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x08, 0x1c, 0x3e, 0x1c, 0x08, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x3e, 0x08, 0x08, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x30, 0x10, 0x20, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3e, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x30, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x02, 0x02, 0x04, 0x08, 0x10, 0x20, 0x20, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x18, 0x24, 0x24, 0x24, 0x24, 0x24, 0x18, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x08, 0x18, 0x08, 0x08, 0x08, 0x08, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x02, 0x1c, 0x20, 0x20, 0x3e, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x02, 0x0c, 0x02, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x04, 0x0c, 0x14, 0x3e, 0x04, 0x04, 0x04, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3e, 0x20, 0x3c, 0x02, 0x02, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x20, 0x20, 0x3c, 0x22, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x3e, 0x02, 0x04, 0x08, 0x10, 0x20, 0x20, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x22, 0x1c, 0x22, 0x22, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x1c, 0x22, 0x22, 0x1e, 0x02, 0x02, 0x1c, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x18, 0x18, 0x08, 0x10, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x04, 0x08, 0x10, 0x20, 0x10, 0x08, 0x04, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x3e, 0x00, 0x3e, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x10, 0x08, 0x04, 0x02, 0x04, 0x08, 0x10, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x18, 0x24, 0x04, 0x08, 0x08, 0x00, 0x08, 0x00, 0x00,

		/* Semigraphics */
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 
		0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 
		0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 
		0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 
		0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
		0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 
		0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 
		0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 
		0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 
		0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 
		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 
		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 
		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xf0, 
		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,

		0x00,0x00,0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00, 0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
		0x00,0x00,0x00,0x00,0x00,0x00, 0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
		0x00,0x00,0x00,0x00,0x00,0x00, 0xff,0xff,0xff,0xff,0xff,0xff,
		0x0f,0x0f,0x0f,0x0f,0x0f,0x0f, 0x00,0x00,0x00,0x00,0x00,0x00,
		0x0f,0x0f,0x0f,0x0f,0x0f,0x0f, 0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
		0x0f,0x0f,0x0f,0x0f,0x0f,0x0f, 0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
		0x0f,0x0f,0x0f,0x0f,0x0f,0x0f, 0xff,0xff,0xff,0xff,0xff,0xff,
		0xf0,0xf0,0xf0,0xf0,0xf0,0xf0, 0x00,0x00,0x00,0x00,0x00,0x00,
		0xf0,0xf0,0xf0,0xf0,0xf0,0xf0, 0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
		0xf0,0xf0,0xf0,0xf0,0xf0,0xf0, 0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
		0xf0,0xf0,0xf0,0xf0,0xf0,0xf0, 0xff,0xff,0xff,0xff,0xff,0xff,
		0xff,0xff,0xff,0xff,0xff,0xff, 0x00,0x00,0x00,0x00,0x00,0x00,
		0xff,0xff,0xff,0xff,0xff,0xff, 0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
		0xff,0xff,0xff,0xff,0xff,0xff, 0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
		0xff,0xff,0xff,0xff,0xff,0xff, 0xff,0xff,0xff,0xff,0xff,0xff
	};

	static struct GfxLayout fontlayout8x12 =
	{
		8,12,	/* 8*12 characters */
		64+64+16,	 /* 64+64+16 characters */
		1,	/* 1 bit per pixel */
		{ 0 },
		{ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 },	/* straightforward layout */
		{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8, 8*8, 9*8, 10*8, 11*8},
		8*12	/* every char takes 12 consecutive bytes */
	};
	struct GfxElement *font;
	static unsigned short colortable[2];
	font = decodegfx(fontdata8x12,&fontlayout8x12);
	if (font)
	{
		/* colortable will be set at run time */
		memset(colortable,0,sizeof(colortable));
		font->colortable = colortable;
		font->total_colors = 2;
	}
	return font;
}

struct GfxElement *build_coco3_font(void)
{
	static unsigned short colortable[2];
	static struct GfxLayout fontlayout8x8 =
	{
		8,8,	/* 8*8 characters */
		128+1,	 /* 128 characters + 1 "underline" */
		1,	/* 1 bit per pixel */
		{ 0 },
		{ 0, 1, 2, 3, 4, 5, 6, 7, },	/* straightforward layout */
		{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
		8*8	/* every char takes 12 consecutive bytes */
	};

	unsigned char buf[129 * 8];
	struct GfxElement *font;

	/* We don't have a font for the low 32 characters */
	memset(buf, 0, 8*32);

	memcpy(&buf[8*32], &dragon_rom[0xf09d - 0x8000], 96 * 8);
	buf[8 * 128 + 0] = 0;
	buf[8 * 128 + 1] = 0;
	buf[8 * 128 + 2] = 0;
	buf[8 * 128 + 3] = 0;
	buf[8 * 128 + 4] = 0;
	buf[8 * 128 + 5] = 0;
	buf[8 * 128 + 6] = 0;
	buf[8 * 128 + 7] = 0xff;

	font = decodegfx(buf,&fontlayout8x8);
	if (font)
	{
		/* colortable will be set at run time */
		memset(colortable,0,sizeof(colortable));
		font->colortable = colortable;
		font->total_colors = 2;
	}
	return font;
}

static int generic_vh_start(size_t maxvram)
{
	display_offset = 0;
	ptape = dragon_tape;
	if ((dfont = build_dragon_font())==NULL)
		return 1; 

	if ((dirtybuffer = malloc (maxvram))==NULL)
		return 1;
	memset(dirtybuffer,1,maxvram);

	pia_config(0, PIA_STANDARD_ORDERING | PIA_8BIT, &dragon_pia_0_intf);
	return 0;
}

int dragon_vh_start(void)
{
	if (generic_vh_start(MAX_VRAM))
		return 1;

	pia_config(1, PIA_STANDARD_ORDERING | PIA_8BIT, &dragon_pia_1_intf);
	pia_reset();

	/* allow short tape leader */
	dragon_rom[0xbdfc - 0x8000] = 4;

	return 0;
}

static int generic_coco_vh_start(size_t maxvram)
{
	if (generic_vh_start(maxvram))
		return 1;

	pia_config(1, PIA_STANDARD_ORDERING | PIA_8BIT, &coco_pia_1_intf);
	pia_reset();
	return 0;
}

int cp400_vh_start(void)
{
	return generic_coco_vh_start(MAX_VRAM);
}

int coco_vh_start(void)
{
	if (generic_coco_vh_start(MAX_VRAM))
		return 1;

	/* allow short tape leader */
	dragon_rom[0xa791 - 0x8000] = 0xff;
	return 0;
}

int coco3_vh_start(void)
{
#if 0	/* HJB unused!? */
    static int initial_palette[] = {
		022, 066, 011, 044, 077, 055, 033, 064,
		000, 022, 000, 077, 000, 022, 000, 064
	};
#endif
    int i;

	if (generic_coco_vh_start(MAX_HIRES_VRAM)) {
		paletteram = NULL;
		return 1;
	}

	if ((coco3font = build_coco3_font())==NULL) {
		paletteram = NULL;
		return 1;
	}

	/* allow short tape leader */
	dragon_rom[0xa791 - 0x8000] = 0xff;

	paletteram = malloc(16 * sizeof(int));
	if (!paletteram)
		return 1;

	memset(paletteram, 0, 16 * sizeof(int));
	for (i = 0; i < 16; i++)
		palette_change_color(i, 0, 0, 0);

	coco3_hires = 0;
	coco3_bordercolor = -1;
	return 0;
}

void dragon_vh_stop(void)
{
	free (dirtybuffer);
	if (dragon_tape) free(dragon_tape);
}

void coco3_vh_stop(void)
{
	dragon_vh_stop();
	if (paletteram) {
		free(paletteram);
		paletteram = NULL;
	}
}

static void coco3_vh_palette_change_color(int color, int data)
{
	palette_change_color(color,
						 (((data >> 4) & 2) | ((data >> 2) & 1)) * 0x55,
						 (((data >> 3) & 2) | ((data >> 1) & 1)) * 0x55,
						 (((data >> 2) & 2) | ((data >> 0) & 1)) * 0x55);
}

static void coco3_vh_setborder(struct osd_bitmap *bitmap, int coco3color, int screenx, int screeny)
{
	int left, right, top, bottom, x, y;
	int borderpen;

	if (coco3_bordercolor != coco3color) {
		borderpen = Machine->pens[16];

		left = (640 - screenx) / 2;
		right = left + screenx;
		top = (225 - screeny) / 2;
		bottom = top + screeny;

		for (y = 0; y < 225; y++) {
			for (x = 0; x < left; x++)
				bitmap->line[y][x] = borderpen;
			for (x = right; x < 640; x++)
				bitmap->line[y][x] = borderpen;
		}
		for (x = left; x < right; x++) {
			for (y = 0; y < top; y++)
				bitmap->line[y][x] = borderpen;
			for (y = bottom; y < 225; y++)
				bitmap->line[y][x] = borderpen;
		}
		osd_mark_dirty(0, 0, 639, top-1, 0);
		osd_mark_dirty(0, bottom, 639, 224, 0);
		osd_mark_dirty(left, 0, right-1, top-1, 0);
		osd_mark_dirty(left, bottom, right-1, 224, 0);

		coco3_bordercolor = coco3color;
		coco3_vh_palette_change_color(16, coco3color);
	}
}

void coco3_palette_w(int offset, int data)
{
	paletteram[offset] = data;
	coco3_vh_palette_change_color(offset, data);

}

static void drawgfx_wf(struct osd_bitmap *dest,const struct GfxElement *gfx,
	unsigned int code,int sx,int sy,
	const struct rectangle *clip,int transparency,int transparent_color, int wf)
{
	if (wf == 1) {
		drawgfx(dest, gfx, code, 0, 0, 0, sx, sy, clip, transparency, transparent_color);
	}
	else {
		/* drawgfxzoom doesnt directly take TRANSPARENCY_NONE */
		if (transparency == TRANSPARENCY_NONE) {
			transparency = TRANSPARENCY_PEN;
			transparent_color = -1;
		}
		drawgfxzoom(dest, gfx, code, 0, 0, 0, sx, sy, clip, transparency, transparent_color, (1<<16) * wf, 1<<16);
	}
}

/* This is a refresh function used by the Dragon/CoCo as well as the CoCo 3 when in lo-res
 * mode.  Internally, it treats the colors like a CoCo 3 and uses the 'metapalette' to
 * translate those colors to the proper palette.
 */
static void generic_vh_screenrefresh(struct osd_bitmap *bitmap, const int *metapalette, UINT8 *vrambase, int basex, int basey, int wf)
{
	int x, y, c, c2, fg, bg, offset, text_color;
	UINT8 *vram, *p1, *p2, *db;
	
	/* clear vblank */
	pia_0_cb1_w (0, 0);
	
	vram = &vrambase[display_offset];
	db = dirtybuffer;
	
	if (pia_vdg_mode & 0x10)
	{
		if (pia_vdg_mode & 0x02)
		{
			/* resolution graphics modes */
			fg = Machine->pens[metapalette[pia_vdg_mode & 0x1 ? 11: 9]];
			bg = Machine->pens[metapalette[pia_vdg_mode & 0x1 ? 10: 8]];
			switch (sam_vdg_mode >> 1)
			{
			case 0x2:
				for (y = 0; y < 192; y++)
					for (x = 0; x < 16; x++)
					{
						if (*db)
						{
							for (c = 0; c < 8; c++) {
								p1 = &bitmap->line[y + basey][x*16+c*2*wf+basex];
								c2 = (*vram & (1<<(7-c))) ? fg : bg;
								*(p1++) = c2;
								*(p1++) = c2;
								if (wf == 2) {
									*(p1++) = c2;
									*(p1++) = c2;
								}
							}
							*db = 0;
							osd_mark_dirty (x*16*wf+basex, y+basey, (x+1)*16*wf-1+basex, y+basey, 0);
						}
						vram++;
						db++;
					}
				break;
			case 0x3:
				for (y = 0; y < 192; y++)
					for (x = 0; x < 32; x++)
					{
						if (*db)
						{
							p1 = &bitmap->line[y+basey][x*8*wf+basex];
							for (c = 0; c < 8; c++) {
								c2 = (*vram & (1<<(7-c))) ? fg : bg;
								*(p1++) = c2;
								if (wf == 2)
									*(p1++) = c2;
							}
							*db = 0;
							osd_mark_dirty (x*8*wf+basex, y+basey, (x+1)*8*wf-1+basex, y+basey, 0);
						}
						vram++;
						db++;
					}
				break;
			default:
				if (errorlog) fprintf (errorlog, "Resolution mode not supported\n");
				break;
			}
		}
		else
		{
			/* color graphics modes */
			offset = pia_vdg_mode & 0x1 ? 4: 0;
			switch (sam_vdg_mode >> 1)
			{
			case 0x2:
				for (y = 0; y < 96; y++)
					for (x = 0; x < 32; x++)
					{
						if (*db)
						{
							p1 = &bitmap->line[y*2+basey][x*8*wf+basex];
							p2 = &bitmap->line[y*2+1+basey][x*8*wf+basex];
							c = Machine->pens[metapalette[offset+(*vram>>6)]];
							*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							c = Machine->pens[metapalette[offset+((*vram>>4)&0x3)]];
							*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							c = Machine->pens[metapalette[offset+((*vram>>2)&0x3)]];
							*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							c = Machine->pens[metapalette[offset+(*vram & 0x3)]];
							*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c; *p2++ = c; *p2++ = c;
							*db = 0;
							osd_mark_dirty (x*8*wf+basex, y*2+basey, (x+1)*8*wf-1+basex, y*2+1+basey, 0);
						}
						vram++;
						db++;
					}
				break;
			case 0x3:
				for (y = 0; y < 192; y++)
					for (x = 0; x < 32; x++)
					{
						if (*db)
						{
							p1 = &bitmap->line[y+basey][x*8*wf+basex];
							c = Machine->pens[metapalette[offset+(*vram>>6)]];
							*p1++ = c; *p1++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c;
							c = Machine->pens[metapalette[offset+((*vram>>4)&0x3)]];
							*p1++ = c; *p1++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c;
							c = Machine->pens[metapalette[offset+((*vram>>2)&0x3)]];
							*p1++ = c; *p1++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c;
							c = Machine->pens[metapalette[offset+(*vram & 0x3)]];
							*p1++ = c; *p1++ = c;
							if (wf == 2)
								*p1++ = c; *p1++ = c;
							*db = 0;
							osd_mark_dirty (x*8*wf+basex, y+basey, (x+1)*8*wf-1+basex, y+basey, 0);
						}
						vram++;
						db++;
					}
				break;
			default:
				if (errorlog) fprintf (errorlog, "Color mode not supported\n");
				break;
			}

		}
	}
	else
	{
		/* Text / semigraphic */
		if (pia_vdg_mode & 0x02)
		{
			offset = pia_vdg_mode & 0x1 ? 4: 0;
			for (y=0; y<16; y++)
				for (x=0; x<32; x++)
				{
					if (*db)
					{
						dfont->colortable[0] = Machine->pens[metapalette[8]];
						dfont->colortable[1] = Machine->pens[metapalette[((*vram >> 6) & 0x3) + offset]];
						drawgfx_wf(bitmap,dfont,64+(*vram &0x3f), x*8+basex, y*12*wf+basey, 0,TRANSPARENCY_NONE,0,wf);
						*db=0;
						osd_mark_dirty (x*8*wf+basex, y*12+basey, (x+1)*8*wf-1+basex, y*12+11+basey, 0);
					}
					vram++;
					db++;
				}
		}
		else
		{
			if (pia_vdg_mode & 0x01)
				text_color = 15;
			else
				text_color = 13;

			for (y=0; y<16; y++)
				for (x=0; x<32; x++)
				{
					if (*db)
					{
						if (*vram & 0x80)
						{
							dfont->colortable[0] = Machine->pens[metapalette[8]];
							dfont->colortable[1] = Machine->pens[metapalette[(*vram >> 4) & 0x7]];

							drawgfx_wf(bitmap,dfont,(*vram & 0x0f)+128,
									x*8*wf+basex, y*12+basey, 0,TRANSPARENCY_NONE,0,wf);
						}
						else
						{
							if (*vram & 0x40)
							{
								dfont->colortable[0] = Machine->pens[metapalette[text_color]];
								dfont->colortable[1] = Machine->pens[metapalette[text_color-1]];
							}
							else
							{
								dfont->colortable[1] = Machine->pens[metapalette[text_color]];
								dfont->colortable[0] = Machine->pens[metapalette[text_color-1]];
							}

							drawgfx_wf(bitmap,dfont,*vram & 0x3f,
									x*8*wf+basex, y*12+basey, 0,TRANSPARENCY_NONE,0,wf);
						}
						*db=0;
						osd_mark_dirty (x*8*wf+basex, y*12+basey, (x+1)*8*wf-1+basex, y*12+11+basey, 0);
					}
					vram++;
					db++;
				}
		}
	}
}

void dragon_vh_screenrefresh(struct osd_bitmap *bitmap, int full_refresh)
{
	static int dragon_metapalette[] = {
		1, 2, 3, 4, 5, 6, 7, 8, 0, 1, 0, 5, 0, 1, 0, 8
	};

	if (full_refresh)
		memset(dirtybuffer,1,MAX_VRAM);
	
	generic_vh_screenrefresh(bitmap, dragon_metapalette, dragon_ram, 0, 0, 1);
}

void coco3_vh_screenrefresh(struct osd_bitmap *bitmap, int full_refresh)
{
	static int coco3_metapalette[] = {
		0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
	};
	static int hires_linesperrow[] = {
		1, 2, 3, 8, 9, 10, 12, 12
	};

	if (coco3_hires) {
		static int blink_status, blink_timer;
		int vidbase, bytesperrow, linesperrow, rows = 0, x, y, basex = 0, basey, wf = 0;
		int use_attr, charsperrow = 0, i, blink_switch = 0, underlined = 0;
		int visualbytesperrow, c, emupixelsperbyte;
		UINT8 *vram, *db;
		UINT8 b;
		unsigned char *p;

		/* clear vblank */
		pia_0_cb1_w (0, 0);

		/* check border */

		vidbase = (coco3_gimevhreg[5] * 0x800) + (coco3_gimevhreg[6] * 8);
		if (coco3_gimevhreg[7] & 0x80) {
			vidbase += coco3_gimevhreg[7] & 0x7f;
			bytesperrow = 256;
		}
		else {
			bytesperrow = 0;
		}

		switch(coco3_gimevhreg[1] & 0x60) {
		case 0:
			rows = 192;
			break;
		case 1:
			rows = 200;
			break;
		case 2:
			rows = 210;
			break;
		case 3:
			rows = 225;
			break;
		}
		basey = (225 - rows) / 2;

		/* check border */
		coco3_vh_setborder(bitmap, coco3_gimevhreg[2] & 0x3f, coco3_gimevhreg[1] & 0x04 ? 640 : 512, rows);

		/* check palette recalc */
		if (palette_recalc() || full_refresh)
			memset(dirtybuffer, 1, MAX_HIRES_VRAM);

		linesperrow = hires_linesperrow[coco3_gimevhreg[0] & 7];

		switch(coco3_gimevhreg[0] & 0x80) {
		case 0x00:	/* Text */
			rows /= 8;
			use_attr = coco3_gimevhreg[1] & 1;
			switch(coco3_gimevhreg[1] & 0x14) {
			case 0x14:
				charsperrow = 80;
				basex = 0;
				wf = 1;
				break;
			case 0x10:
				charsperrow = 64;
				basex = 64;
				wf = 1;
				break;
			case 0x04:
				charsperrow = 40;
				basex = 0;
				wf = 2;
				break;
			case 0x00:
				charsperrow = 32;
				basex = 64;
				wf = 2;
				break;
			}

			if (!bytesperrow)
				bytesperrow = charsperrow * (use_attr + 1);

			if (use_attr) {
				/* Resolve blink */
				/* TODO: This needs to be adjusted to a true timer; not a counter */
				blink_switch = (++blink_timer > 3);
				
				if (blink_switch) {
					blink_timer = 0;
					blink_status = !blink_status;
				}
			}
			else {
				/* Set colortable to be default */
				coco3font->colortable[0] = Machine->pens[0];
				coco3font->colortable[1] = Machine->pens[1];
				underlined = 0;
			}

			for (y = 0; y < rows; y++) {
				i = y * bytesperrow;
				vram = &RAM[(vidbase + i) & 0x7FFFF];
				db = dirtybuffer + i;

				for (x = 0; x < charsperrow; x++) {
					if (db[0] || (use_attr && (db[1] || (blink_switch && (vram[1] & 0x80))))) {
						b = *vram & 0x7f;

						if (use_attr) {
							coco3font->colortable[0] = Machine->pens[vram[1] & 7];
							coco3font->colortable[1] = Machine->pens[8 + ((vram[1] >> 3) & 7)];
							
							/* Are we blinking? */
							if (blink_status && (vram[1] & 0x80)) {
								b = 0;
								underlined = 0;
							}
							else {
								/* Are we underlined? */
								underlined = vram[1] & 0x40;
							}
						}

						drawgfx_wf(bitmap, coco3font, b, x*8*wf+basex, y*8+basey, 0, TRANSPARENCY_NONE, 0, wf);
						if (underlined)
							drawgfx_wf(bitmap, coco3font, 128, x*8*wf+basex, y*8+basey, 0, TRANSPARENCY_PEN, 0, wf);
						osd_mark_dirty(x*8*wf+basex, y*8+basey, (x+1)*8*wf-1+basex, y*8+7+basey, 0);

						db[0] = 0;
						if (use_attr) 
							db[1] = 0;
					}
					vram++;
					db++;
					vram += use_attr;
					db += use_attr;

					if (vram > &RAM[0x80000])
						vram -= 0x80000;
				}
			}
			break;

		case 0x80:	/* Graphics */
			visualbytesperrow = 16 << ((coco3_gimevhreg[1] & 0x18) >> 3);
			switch(coco3_gimevhreg[1] & 3) {
			case 0:
				/* Two colors */
				wf = 64 / visualbytesperrow;
				break;
			case 1:
				/* Four colors */
				wf = 128 / visualbytesperrow;
				break;
			case 2:
			case 3:
				/* Sixteen colors */
				wf = 256 / visualbytesperrow;
				break;
			}

			emupixelsperbyte = 512 / visualbytesperrow;

			if (coco3_gimevhreg[1] & 0x04) {
				visualbytesperrow |= (visualbytesperrow / 4);
				basex = 0;
			}
			else {
				basex = 64;
			}

			if (!bytesperrow)
				bytesperrow = visualbytesperrow;

			for (y = 0; y < rows; y++) {
				i = y * bytesperrow;
				vram = &RAM[(vidbase + i) & 0x7FFFF];
				db = dirtybuffer + i;
				p = bitmap->line[y + basey];

				for (x = 0; x < visualbytesperrow; x++) {
					if (*db) {
						switch(coco3_gimevhreg[1] & 3) {
						case 0:
							/* Two colors */
							for (i = 0; i < 8; i++) {
								c = Machine->pens[(*vram & (1<<(7-i))) ? 1 : 0];
								switch(wf) {
								case 4:	*(p++) = c;	*(p++) = c;
								case 2:	*(p++) = c;
								case 1:	*(p++) = c;	break;
								}
							}
							osd_mark_dirty (x*8*wf, y+basey, (x+1)*8*wf-1, y+basey, 0);
							break;
						case 1:
							/* Four colors */
							for (i = 0; i < 4; i++) {
								c = Machine->pens[(*vram & (3<<(6-i*2))) >> (6-i*2)];
								switch(wf) {
								case 4:	*(p++) = c;	*(p++) = c;
								case 2:	*(p++) = c;
								case 1:	*(p++) = c;	break;
								}
							}
							osd_mark_dirty (x*4*wf, y+basey, (x+1)*4+wf, y+basey, 0);
							break;
						case 2:
						case 3:
							/* Sixteen colors */
							c = Machine->pens[(*vram>>4) & 0x0F];
							switch(wf) {
							case 4:	*(p++) = c;	*(p++) = c;
							case 2:	*(p++) = c; *(p++) = c;	break;
							}
							c = Machine->pens[(*vram>>0) & 0x0F];
							switch(wf) {
							case 4:	*(p++) = c;	*(p++) = c;
							case 2:	*(p++) = c; *(p++) = c;	break;
							}
							osd_mark_dirty (x*2*wf, y+basey, (x+1)*2*wf-1, y+basey, 0);
							break;
						}
						*db = 0;
					}
					else {
						p += emupixelsperbyte;
					}
					vram++;
					db++;

					if (vram > &RAM[0x80000])
						vram -= 0x80000;
				}
			}
			break;
		}
	}
	else {
		coco3_vh_setborder(bitmap, (pia_vdg_mode & 0x10) ? ((pia_vdg_mode & 0x1) ? 077 : 022) : 000, 512, 192);
		if (palette_recalc() || full_refresh)
			memset(dirtybuffer, 1, MAX_VRAM);
		generic_vh_screenrefresh(bitmap, coco3_metapalette, &RAM[0x70000], 64, 16, 2);
	}
}

int dragon_interrupt(void)
{
	pia_0_cb1_w (0, 1);
	return ignore_interrupt();
}

static void d_pia1_pb_w(int offset, int data)
{
	if ((data >> 3) != pia_vdg_mode)
	{
		pia_vdg_mode = data >> 3;
		memset(dirtybuffer,1,MAX_VRAM);
	}

}

static void d_pia1_pa_w(int offset, int data)
{
	d_dac = data & 0xfa;
	if (sound_mux)
		DAC_data_w(0,d_dac);
}

static int d_pia0_ca1_r(int offset)
{
	return 0; 
}

static int d_pia1_cb1_r(int offset)
{
	return dragon_cart_inserted;
}

static void d_pia1_cb2_w(int offset, int data)
{
	sound_mux = data;
}

static void d_pia0_cb2_w(int offset, int data)
{
	joystick = data;
}

static void d_pia1_ca2_w(int offset, int data)
{
	if (tape_motor ^ data)
	{
		/* speed up tape reading */
		dragon_ram[0x0093] = 2;	 
		dragon_ram[0x0092] = 3;

		if (data == 0)
		{
			ptape--;
			ptape[0] = 0x55; /* insert sync byte */
		}
		tape_motor = data;
	}
}

static void coco_pia1_ca2_w(int offset, int data)
{
	if (tape_motor ^ data)
	{
		/* speed up tape reading */
		dragon_ram[0x008f] = 3;	 
		dragon_ram[0x0091] = 2;	 

		if (data == 0)
		{
			ptape--;
			ptape[0] = 0x55; /* insert sync byte */
		}
		tape_motor = data;
	}
}

static void d_pia0_ca2_w(int offset, int data)
{
	joystick_axis = data;
}

static int d_pia0_pa_r(int offset)
{
	int porta=0x7f;

	if ((input_port_0_r(0) | pia1_pb) != 0xff) porta &= ~0x01;
	if ((input_port_1_r(0) | pia1_pb) != 0xff) porta &= ~0x02;
	if ((input_port_2_r(0) | pia1_pb) != 0xff) porta &= ~0x04;
	if ((input_port_3_r(0) | pia1_pb) != 0xff) porta &= ~0x08;
	if ((input_port_4_r(0) | pia1_pb) != 0xff) porta &= ~0x10;
	if ((input_port_5_r(0) | pia1_pb) != 0xff) porta &= ~0x20;
	if ((input_port_6_r(0) | pia1_pb) != 0xff) porta &= ~0x40;
	if (d_dac <= (joystick_axis? input_port_8_r(0): input_port_7_r(0)))
		porta |= 0x80;
	porta &= ~input_port_9_r(0);

	return porta;
}

static int d_pia1_pa_r(int offset)
{
	static int bit=7, bitc=0, *state;
	static int lo[]={1,1,0,0};
	static int hi[]={1,0};

	if (ptape && tape_motor)
	{
		if (bitc == 0)
		{
			if (bit < 0)
			{
				bit = 7;
				if (ptape - dragon_tape < dragon_tapesize)
					ptape++;
			}

			if ((*ptape >> (7-bit)) & 0x01)
			{
				state = hi;
				bitc = 2;
			}
			else
			{
				state = lo;
				bitc = 4;
			}
			bit--;
		}
		bitc--;
		return (state[bitc]);
	}
	return 1;
}

static void d_pia0_pb_w(int offset, int data)
{
	pia1_pb = data;
}

static void d_pia0_irq_b(int state)
{
	cpu_set_irq_line(0, M6809_IRQ_LINE, state);
}

void dragon_sam_display_offset(int offset, int data)
{
	UINT16 d_offset = display_offset;

	if (offset & 0x01)
		d_offset |= 0x01 << (offset/2 + 9);
	else
		d_offset &= ~(0x01 << (offset/2 + 9));

	if (d_offset != display_offset)
	{
		memset(dirtybuffer,1,MAX_VRAM);
		display_offset = d_offset;
	}
}

void dragon_sam_vdg_mode(int offset, int data)
{
	if (offset & 0x01)
		sam_vdg_mode |= 0x01 << (offset/2);
	else
		sam_vdg_mode &= ~(0x01 << (offset/2));
}

static void generic_ram_w(int offset, int data, UINT8 *vrambase)
{
	if ((offset >= display_offset) && (offset < display_offset + MAX_VRAM))
		if (vrambase[offset] != data)
			dirtybuffer[offset - display_offset] = 1;
}

void dragon_ram_w (int offset, int data)
{
	generic_ram_w(offset, data, dragon_ram);
	dragon_ram[offset] = data;
}

static void coco3_ram_w(int offset, int data, int block)
{
	extern int coco3_mmu_translate(int _block, int _offset);
	int vidbase;
	unsigned int vidbasediff;

	offset = coco3_mmu_translate(block, offset);

	if (coco3_hires) {
		vidbase = (coco3_gimevhreg[5] * 0x800) + (coco3_gimevhreg[6] * 8);
		vidbasediff = (unsigned int) (offset - vidbase) & 0x7ffff;
		if (vidbasediff < MAX_HIRES_VRAM)
			dirtybuffer[vidbasediff] = 1;
	}
	else {
		if (offset >= 0x70000)
			generic_ram_w(offset - 0x70000, data, &RAM[0x70000]);
	}

	RAM[offset] = data;
}

void coco3_ram_b1_w (int offset, int data)
{
	coco3_ram_w(offset, data, 0);
}
void coco3_ram_b2_w (int offset, int data)
{
	coco3_ram_w(offset, data, 1);
}
void coco3_ram_b3_w (int offset, int data)
{
	coco3_ram_w(offset, data, 2);
}
void coco3_ram_b4_w (int offset, int data)
{
	coco3_ram_w(offset, data, 3);
}
void coco3_ram_b5_w (int offset, int data)
{
	coco3_ram_w(offset, data, 4);
}
void coco3_ram_b6_w (int offset, int data)
{
	coco3_ram_w(offset, data, 5);
}
void coco3_ram_b7_w (int offset, int data)
{
	coco3_ram_w(offset, data, 6);
}
void coco3_ram_b8_w (int offset, int data)
{
	coco3_ram_w(offset, data, 7);
}

int coco3_gimevh_r(int offset)
{
	return coco3_gimevhreg[offset];
}

void coco3_gimevh_w(int offset, int data)
{
	/* Features marked with '!' are not yet implemented */
	coco3_gimevhreg[offset] = data;
	switch(offset) {
	case 0:
		/*	$FF98 Video Mode Register
		 *		  Bit 7 BP 0 = Text modes, 1 = Graphics modes
		 *		  Bit 6 Unused
		 *		! Bit 5 BPI Burst Phase Invert (Color Set)
		 *		! Bit 4 MOCH 1 = Monochrome on Composite
		 *		! Bit 3 H50 1 = 50 Hz power, 0 = 60 Hz power
		 *		  Bits 0-2 LPR Lines per row
		 */
		coco3_bordercolor = -1;	/* force border to redraw */
		memset(dirtybuffer,1,MAX_HIRES_VRAM);
		break;

	case 1:
		/*	$FF99 Video Resolution Register
		 *		  Bit 7 Undefined
		 *		  Bits 5-6 LPF Lines per Field (Number of Rows)
		 *		  Bits 2-4 HRES Horizontal Resolution
		 *		  Bits 0-1 CRES Color Resolution
		 */
		coco3_bordercolor = -1;	/* force border to redraw */
		memset(dirtybuffer,1,MAX_HIRES_VRAM);
		break;

	case 2:
		/*	$FF9A Border Register
		 *		  Bits 6,7 Unused
		 *		  Bits 0-5 BRDR Border color	
		 */
		break;

	case 4:
		/*	$FF9C Vertical Scroll Register
		 *		  Bits 4-7 Reserved
		 *		  Bits 0-3 VSC Vertical Scroll bits
		 */
		memset(dirtybuffer,1,MAX_HIRES_VRAM);
		break;

	case 5:
	case 6:
		/*	$FF9D,$FF9E Vertical Offset Registers
		 */
		memset(dirtybuffer,1,MAX_HIRES_VRAM);
		break;

	case 7:
		/*	$FF9F Horizontal Offset Register
		 *		  Bit 7 HVEN Horizontal Virtual Enable
		 *		  Bits 0-6 X0-X6 Horizontal Offset Address
		 */
		memset(dirtybuffer,1,MAX_HIRES_VRAM);
		break;
	}
}

void coco3_vh_sethires(int hires)
{
	if (hires != coco3_hires) {
		coco3_hires = hires;
	}
}

